﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("BestTrainingXCSSolutionAnalyzer", "")]
  [StorableClass]
  public sealed class BestTrainingXCSSolutionAnalyzer : XCSSolutionAnalyzer {
    private const string BestTrainingSolutionName = "Best Training Solution";


    [StorableConstructor]
    private BestTrainingXCSSolutionAnalyzer(bool deserializing) : base(deserializing) { }
    private BestTrainingXCSSolutionAnalyzer(BestTrainingXCSSolutionAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BestTrainingXCSSolutionAnalyzer(this, cloner);
    }
    public BestTrainingXCSSolutionAnalyzer()
      : base() {
    }

    protected override void UseCurrentXCSSolution(IXCSSolution xcsSolution) {
      if (!Results.ContainsKey(BestTrainingSolutionName)) {
        Results.Add(new Result(BestTrainingSolutionName, (IXCSSolution)xcsSolution.Clone()));
      } else {
        if (xcsSolution.TrainingAccuracy > ((IXCSSolution)Results[BestTrainingSolutionName].Value).TrainingAccuracy) {
          Results[BestTrainingSolutionName].Value = (IXCSSolution)xcsSolution.Clone();
        }
      }
    }
  }
}
